require 'spec_helper'

describe Almicube::Aggregator::SumAggregator do
  before(:all) { @connection = Redis.new }
  before do
    @connection.flushdb
    Almicube::Config.config.stub(:connection) { @connection }
  end

  describe "#aggregate" do
    let(:key) { "sample" }
    let(:aggregator) { described_class.new }
    let(:attribute) { :score }

    context "with data ranking" do
      [10, 100, 300].each do |size|
        describe "and #{size} records" do
          let(:ranking) { double( records: records, selector: nil, data?: true, key: key, attribute_name: attribute ) }
          let(:records) { size.times.map { |i| double( to_param: i+1, score: 100, access: (i + 2) * 12 ) } }

          before { aggregator.ranking = ranking }

          it { expect(records.length).to eq size }
          it { expect{ aggregator.aggregate }.not_to raise_error }
          it { expect{ aggregator.aggregate }.to change{ @connection.zcard(key) }.by size }

          describe "for score" do
            before { aggregator.aggregate }
            it { records.each { |record| expect(@connection.zscore(key, record.to_param)).to eq 100 } }
          end

          describe "for access" do
            let(:attribute) { :access }
            before { aggregator.aggregate }
            it { records.each { |record| expect(@connection.zscore(key, record.to_param)).to eq (record.to_param+1)*12} }
          end
        end
      end
    end

    context "with sub ranking" do
      let(:size) { 10 }
      let(:keys) { ["sample1", "sample2"] }
      let(:selector) { double( interstore: nil, before_aggregate: nil ) }
      let(:ranking) { double( bundled_keys: keys, selector: selector, data?: false, key: key ) }

      before do
        keys.each { |key| size.times { |i| @connection.zadd(key, 10, i) } }
        aggregator.ranking = ranking
      end

      it { expect{ aggregator.aggregate }.to change{ @connection.zcard(key) }.by size }

      describe "for score" do
        before { aggregator.aggregate }
        it { size.times { |i| expect(@connection.zscore(key, i)).to eq 20 } }
      end
    end
  end
end
