require 'spec_helper'

describe Almicube::Config do
  before do
    File.stub(:exists?).and_return(true)
    YAML.stub(:load_file).and_return(test_config)
    described_class.reset
  end

  describe "#connection" do
    let(:default_param) { { url: "redis://0.0.0.0:6380/1" } }
    let(:development_param) { { url: "redis://0.0.0.0:6380/15" } }
    let(:test_config) do
      {
        connection: {
          default: default_param,
          development: development_param
        }
      }
    end

    context "with default connection" do
      before { Redis.should_receive(:new).with( default_param ) { true } }
      it { described_class.config.connection }
    end

    context "with actual connection" do
      before { Redis.should_receive(:new).with( development_param ) { true } }
      it { described_class.config.connection :development }
    end

    context "with no-exists connection" do
      before { Redis.stub(:new) }
      it { expect{ described_class.config.connection :noexists }.to raise_error }
    end

    context "already setting up default connection" do
      let(:redis) { double("redis") }
      before do
        Redis.stub(:new).and_return(redis)
        described_class.config.connection
      end
      it { expect(described_class.config.connection).to eq redis }
    end
  end
end
