require 'spec_helper'

describe Almicube::Ranking::Base do
  before { @connection = Redis.new }

  def setup_datastore(key)
    @connection.del key
    @connection.zadd key, 10, 1
    @connection.zadd key, 20, 2
  end

  describe "#bundled_keys" do
    let(:bundler) { double( keys: ["1", "2"], :"ranking=" => nil ) }
    let(:ranking) { described_class.new bundler: bundler }
    it { expect(ranking.bundled_keys).to eq ["1", "2"] }
  end

  describe "#page" do
    let(:ranking) { described_class.new class_name: MockModel }
    before do
      ranking.stub(:key) { "sample" }
      setup_datastore "sample"
      Almicube::Config.config.stub(:connection) { @connection }
      MockModel.should_receive(:where).with("id IN (?)", ["2", "1"]) { [] }
    end
    it { ranking.page 1 }
  end

  describe "#data?" do
    let(:ranking) { described_class.new }
    it { expect(ranking).not_to be_data }
  end

  describe "score" do
    let(:ranking) { described_class.new class_name: MockModel }
    before do
      ranking.stub(:key) { "sample" }
      setup_datastore "sample"
      Almicube::Config.config.stub(:connection) { @connection }
    end
    it { expect(ranking.score(1)).to eq 10 }
    it { expect(ranking.score(2)).to eq 20 }
  end

  describe "rank" do
    let(:ranking) { described_class.new class_name: MockModel }
    before do
      ranking.stub(:key) { "sample" }
      setup_datastore "sample"
      Almicube::Config.config.stub(:connection) { @connection }
    end
    it { expect(ranking.rank(1)).to eq 2 }
    it { expect(ranking.rank(2)).to eq 1 }
  end

  describe "#aggregate" do
    context "raises exception from aggregator" do
      let(:ranking) { described_class.new class_name: MockModel }
      before { ranking.stub(:aggregate!).and_raise "exception" }
      it { expect(ranking.aggregate).to eq false }
    end
  end
end
