require 'spec_helper'

describe Almicube::Ranking::SubRanking do
  let(:data_selector) { MockSelector.new }
  let(:data_ranking) { Almicube::Ranking::DataRanking.new( selector: data_selector ) }

  describe "#key" do
    let(:ranking) { described_class.new(data_ranking, bundler: bundler) }
    let(:bundler) { double( bundle: :bundled, :"ranking=" => nil ) }

    it { expect(ranking.key).not_to match /:data:/ }
    it { expect(ranking.key).to match /:built:/ }
    it { expect(ranking.key).to match /:bundled$/ }
  end

  describe "#selector" do
    context "uses optional selector" do
      let(:selector) { double( records: [], :"ranking=" => nil ) }
      let(:ranking) { described_class.new(data_ranking, selector: selector) }
      it { expect(ranking.selector).to eq selector }
    end

    context "uses selector on data ranking" do
      let(:ranking) { described_class.new(data_ranking) }
      it { expect(ranking.selector).to eq data_selector }
    end
  end

  describe "#records" do
    let(:selector) { double( records: ["a", "b"], :"ranking=" => nil ) }
    let(:ranking) { described_class.new(data_ranking, selector: selector) }
    it { expect(ranking.records).to eq selector.records }
  end
end
