require 'spec_helper'
require 'mocks/model'

describe Almicube::Selector::MethodSelector do
  describe "#initialize" do
    it { expect{ described_class.new }.to raise_error }
    it { expect{ described_class.new class_name: "UnknownClassName" }.to raise_error }
    it { expect{ described_class.new accessor: :invalid, class_name: MockModelForSelector }.to raise_error }
    it { expect{ described_class.new accessor: :all, class_name: MockModelForSelector }.to_not raise_error }
  end

  describe "#records" do
    context "setted accessor method is all" do
      let(:selector) { described_class.new accessor: :all, class_name: MockModelForSelector }
      let(:records) { [double(), double()] }
      before { MockModelForSelector.should_receive(:all).and_return(records) }
      it { expect(selector.records).to eq records }
    end
  end

  describe "#count" do
    context "setted counter method is count" do
      let(:selector) { described_class.new counter: :count, class_name: MockModelForSelector }
      let(:count) { 100 }
      before { MockModelForSelector.should_receive(:count).and_return(count) }
      it { expect(selector.count).to eq count }
    end
  end

  describe "default accessor" do
    let(:selector) { described_class.new class_name: MockModelForSelector }
    it { expect(selector.accessor).to eq :all }
  end

  describe "default counter" do
    let(:selector) { described_class.new class_name: MockModelForSelector }
    it { expect(selector.counter).to eq :count }
  end
end
